"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var clone = require("clone");
var React = require("react");
var hoistNonReactStatic = require("hoist-non-react-statics");
var invariant = require("invariant");
var predicates_1 = require("./predicates");
function cloneState(state) {
    if (predicates_1.default(state)) {
        return state;
    }
    return clone(state);
}
exports.cloneState = cloneState;
function getStateDelegate(state, getState, name) {
    return function () {
        if (!predicates_1.default(state)) {
            return getState()[name];
        }
        return getState().get(name);
    };
}
exports.getStateDelegate = getStateDelegate;
function forwardComponent(forwardRef, Component, WrappedComponent, name) {
    Component.WrappedComponent = WrappedComponent;
    var wrappedComponentName = WrappedComponent.displayName || WrappedComponent.name || 'Component';
    var displayName = name + "(" + wrappedComponentName + ")";
    Component.displayName = displayName;
    if (forwardRef) {
        var forwarded = React.forwardRef(function (props, ref) {
            return React.createElement(Component, __assign({}, props, { forwardedRef: ref }));
        });
        forwarded.displayName = displayName;
        forwarded.WrappedComponent = WrappedComponent;
        return hoistNonReactStatic(forwarded, WrappedComponent);
    }
    return hoistNonReactStatic(Component, WrappedComponent);
}
exports.forwardComponent = forwardComponent;
function checkModelType(model, name, modelTypeName) {
    var instanceCon = modelTypeName[name];
    if (instanceCon == null) {
        modelTypeName[name] = model.constructor;
    }
    else {
        invariant(instanceCon === model.constructor, "Different Model should not use the same model name, Please check name: " + name);
    }
    invariant(Object.prototype.toString.call(model.state) === '[object Object]', 'The shape of state must be an object');
}
exports.checkModelType = checkModelType;
